/*
 mqtt_ethernet_test3:
 MQTT - verteilt TRX-Frequenz via mosquitto-MQTT-Broker
 Hardware: Arduino UNO und Arduino Ethernet Shield, Yaesu FT857D (CAT 4800 Baud)
 -------------------------------------------------------------------------------

 Empfängt TRX-Frequenz vom Yaesu FT857D über SoftwareSerial RX = Pin 2, TX = Pin 3
 Verbindet mit MQTT-Broker "test.mosquitto.org" auf Port 1883
 ->>> alternativ ein MQTT-Broker im HAMNET: mqtt.hc.r1.ampr.org
 Veröffentlicht aktuelle TRX-Frequenz alle 5 Sekunden unter Topic "TRX_Freq"

 Diese Veröffentlichung kann weltweit über einen MQTT-Client 
 unter dem Topic TRX_Freq abgerufen (abonniert) werden.
 
*/

#include <SoftwareSerial.h>
#include <FT857D.h>             // the file FT857D.h has a lot of documentation to make using the library easier

#include <SPI.h>
#include <Ethernet.h>
#include <PubSubClient.h>

// Update these with values suitable for your network.
byte mac[]    = { 0x90, 0xA2, 0xDA, 0x00, 0x9D, 0xB5 };
IPAddress ip(192, 168, 178, 242);
IPAddress server(192, 168, 178, 1);

char cstr[12];
unsigned long num;
unsigned long lastmillis;
float freq;
      
void callback(char* topic, byte* payload, unsigned int length) {
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  for (int i=0;i<length;i++) {
    Serial.print((char)payload[i]);
  }
  Serial.println();
}

FT857D myFT857;                       // define "myFT857" so that we may pass CAT commands
EthernetClient ethClient;
PubSubClient client(ethClient);

void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) 
  {
    Serial.print("Verbinde mit MQTT Broker...");

    // Parameter der nächsten Zeile: ClientID, Topic, QoS, Retain, LastWill&Testament
    if (client.connect("arduinoClient","TRX_status",0,true,"offline")) 
    {
      Serial.println("connected");
      
      // Wenn mit MQTT-Broker verbunden, Frequenz und Status senden...
      // ---------------------------------------------------------------
      num = myFT857.getFreqMode();                      // Frequenz in Variable num ablegen
      Serial.print("num: ");Serial.println(num);        // Kontrollanzeige auf Seriallem Monitor
      freq = (float) num / 100000;                      // in MHz wandeln
      dtostrf(freq,3, 6, cstr);                         // in Char-Array kopieren
      Serial.print("cstr: ");Serial.println(cstr);      // Kontrollausgabe
      client.publish("TRX_Freq", cstr, true);           // senden zum MQTT-Broker, true = retain

      // Wir können auch den aktuellen Status des TRX senden...
      //boolean publish(const char* topic, const char* payload, boolean retained);
      client.publish("TRX_status","online",true);       //true sendet die Nachricht retained
    } 
    else 
    {
      Serial.print("nicht verbunden, rc=");
      Serial.print(client.state());
      Serial.println(" versuche in 10 Sek. erneut...");
      // Wait 10 seconds before retrying
      delay(10000);
    }
  } // while connected...
}

void setup()
{
  Serial.begin(115200);               // Baudrate für den Seriellen Monitor in der Arduino IDE
  SoftwareSerial userSerial(2,3);     // rx-Pin,tx-Pin am UNO
  myFT857.begin(4800);                // CAT mit 4800 Baud

  client.setServer("test.mosquitto.org", 1883);
  client.setCallback(callback);

  Ethernet.begin(mac, ip);
  // Allow the hardware to sort itself out
  delay(1500);
}

void loop()
{
  if (!client.connected() && millis() > lastmillis + 5000) 
  {
    reconnect();
    lastmillis = millis();
  }
  client.loop();
  
}
